@@ -1,4 +1,4 @@
-/* Self test, based on examples/simple/ssh2.c. */
+/* Self test, based on examples/ssh2.c. */
 
 #include "libssh2_config.h"
 #include <libssh2.h>
@@ -39,17 +39,23 @@ int main(int argc, char *argv[])
     char *userauthlist;
     LIBSSH2_SESSION *session;
     LIBSSH2_CHANNEL *channel;
-#ifdef WIN32
-    WSADATA wsadata;
-
-    WSAStartup(MAKEWORD(2,0), &wsadata);
-#endif
     const char *pubkeyfile="etc/user.pub";
     const char *privkeyfile="etc/user";
     const char *username="username";
     const char *password="password";
     int ec = 1;
 
+#ifdef WIN32
+    WSADATA wsadata;
+    int err;
+
+    err = WSAStartup(MAKEWORD(2,0), &wsadata);
+    if (err != 0) {
+        fprintf(stderr, "WSAStartup failed with error: %d\n", err);
+        return -1;
+    }
+#endif
+
     (void)argc;
     (void)argv;
 
@@ -59,7 +65,7 @@ int main(int argc, char *argv[])
     if (getenv ("PRIVKEY"))
       privkeyfile = getenv ("PRIVKEY");
 
-    if (getenv ("PRIVKEY"))
+    if (getenv ("PUBKEY"))
       pubkeyfile = getenv ("PUBKEY");
 
     hostaddr = htonl(0x7F000001);
@@ -74,7 +80,7 @@ int main(int argc, char *argv[])
     if (connect(sock, (struct sockaddr*)(&sin),
                 sizeof(struct sockaddr_in)) != 0) {
         fprintf(stderr, "failed to connect!\n");
-        return -1;
+        return 1;
     }
 
     /* Create a session instance and start it up
@@ -83,16 +89,16 @@ int main(int argc, char *argv[])
     session = libssh2_session_init();
     if (libssh2_session_startup(session, sock)) {
         fprintf(stderr, "Failure establishing SSH session\n");
-        return -1;
+        return 1;
     }
 
     /* At this point we havn't authenticated,
      * The first thing to do is check the hostkey's fingerprint against our known hosts
      * Your app may have it hard coded, may go to a file, may present it to the user, that's your call
      */
-    fingerprint = libssh2_hostkey_hash(session, LIBSSH2_HOSTKEY_HASH_MD5);
+    fingerprint = libssh2_hostkey_hash(session, LIBSSH2_HOSTKEY_HASH_SHA1);
     printf("Fingerprint: ");
-    for(i = 0; i < 16; i++) {
+    for(i = 0; i < 20; i++) {
         printf("%02X ", (unsigned char)fingerprint[i]);
     }
     printf("\n");
